/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * gui-progress.c: Progress window
 *
 * Copyright (C) 2000, 2001 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gnome.h>
#include <gui.h>
#include <gal/util/e-unicode-i18n.h>

#include "gui-download.h"

GUIInstallType install_type;

char *progress_icons[] = {
    "progress-download.png",
    "progress-verify.png",
    "progress-install.png",
    "progress-finished.png",
    "progress-finished.png"
};

#define H "#ffffff"
#define A "#000000"
#define P "#a3a3a3"

static char *
html_color(GUIInstallProgress colorize, GUIInstallProgress current, 
           gboolean hide)
{
    char *ACTIVE = A;
    char *PASSED = P;
    char *HIDDEN = hide ? H : P;

    char *color_table[5][3] = {
                                     /* D */ /* V */ /* I */
        /* PROGRESS_DOWNLOAD */    { ACTIVE, HIDDEN, HIDDEN },
        /* PROGRESS_VERIFY   */    { PASSED, ACTIVE, HIDDEN },
        /* PROGRESS_TRANSACT  */   { PASSED, PASSED, ACTIVE },
        /* PROGRESS_WAIT_FOR_IT */ { PASSED, PASSED, PASSED },
        /* PROGRESS_FINISHED */    { PASSED, PASSED, PASSED }
    };

    return color_table[current][colorize];
} /* html_color */

static void
gtk_progress_sucks_my_ass(GtkProgress *progress)
{
    GtkWidget *widget;

    g_return_if_fail (progress != NULL);
    g_return_if_fail (GTK_IS_PROGRESS (progress));

    if (progress->offscreen_pixmap)
        gdk_pixmap_unref(progress->offscreen_pixmap);

    if (GTK_WIDGET_REALIZED(progress)) {
        widget = GTK_WIDGET(progress);

        progress->offscreen_pixmap = gdk_pixmap_new(
            widget->window, widget->allocation.width,
            widget->allocation.height, -1);
        GTK_PROGRESS_CLASS(GTK_OBJECT(progress)->klass)->paint(progress);
    }
    else
        progress->offscreen_pixmap = NULL;
}

static void
rc_gui_set_progress_install(GUIInstallProgress step, int total, int done, 
                            char *name)
{
    GtkHTMLStream *stream;
    char *h;
    const char *h1;
    GtkWidget *single, *tot;

    static int download_done;
    static int verify_done;
    static int transact_done;

    static int download_total;
    static int verify_total;
    static int transact_total;

    single = gtk_object_get_data(
        GTK_OBJECT(appinfo->download_html), "single_progress");
    if (single->parent) {
        gtk_widget_ref(single);
        gtk_container_remove(GTK_CONTAINER(single->parent), single);
    }

    tot = gtk_object_get_data(
        GTK_OBJECT(appinfo->download_html), "total_progress");
    if (tot->parent) {
        gtk_widget_ref(tot);
        gtk_container_remove(GTK_CONTAINER(tot->parent), tot);
    }

    switch (step) {
    case PROGRESS_DOWNLOAD:
        download_done = done;
        if (total) {
            download_total = total;
            /* Reset the others */
            verify_done = 0;
            transact_done = 0;
            verify_total = 0;
            transact_total = 0;
        }
        break;
    case PROGRESS_VERIFY:
        verify_done = done;
        if (total) {
            verify_total = total;
        }
        break;
    case PROGRESS_TRANSACT:
        transact_done = done;
        if (total) {
            transact_total = total;
        }
        break;
    case PROGRESS_WAIT_FOR_IT:
    case PROGRESS_FINISHED:
        break;
    default:
        g_assert_not_reached();
    }

    stream = gtk_html_begin(appinfo->download_html);
    h = g_strdup_printf(
        "<html><body marginwidth=0 marginheight=0 bgcolor=#e5e3e3>"
        "<table width=100%% cellspacing=0 cellpadding=2 bgcolor=#ffffff>"
        "<tr valign=top>"
        "<td rowspan=4 width=100>"
        "<img src=\"file://"SHAREDIR"/%s\" width=100 height=100>"
        "</td>"
        "<td width=120>"
        "<font size=\"+1\" color=%s><b>%s</b></font></td>" /* "Downloading" */
        "<td><font color=%s>%s</font>"
        "<font color=%s>"
        "<table cellspacing=0 cellpadding=0><tr>"
        "<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>"
        "<td width=40>%s:</td>"
        "<td width=30><b>%d</b></td>"
        "<td width=85>&nbsp;&nbsp;&nbsp;&nbsp;%s:</td>"
        "<td width=30><b>%d</b></td>"
        "<td width=130>&nbsp;&nbsp;&nbsp;&nbsp;%s:</td>"
        "<td width=30><b>%d%%</b></td>"
        "</tr></table>"
        "</font>"
        "</td>"
        "<td align=right width=66>"
        "<a href=\"updater:download_log\">"
        "<img src=\"file://"SHAREDIR"/info-off.png\" width=66 height=30 "
        "     border=no></a></td>"
        "</tr>"
        "<tr valign=top>"
        "<td width=100>"
        "<font size=\"+1\" color=%s><b>%s</b></font></td>"
        "<td><font color=%s>%s</font>"
        "<font color=%s><table cellspacing=0 cellpadding=0><tr>"
        "<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>"
        "<td width=40>%s:</td>"
        "<td width=30><b>%d</b></td>"
        "<td width=85>&nbsp;&nbsp;&nbsp;&nbsp;%s:</td>"
        "<td width=30><b>%d</b></td>"
        "<td width=130>&nbsp;&nbsp;&nbsp;&nbsp;%s:</td>"
        "<td width=30><b>%d%%</b></td>"
        "</tr></table></font></td>"
        "<td align=right width=66>"
        "<a href=\"updater:verify_log\">"
        "<img src=\"file://"SHAREDIR"/info-off.png\" width=66 height=30 "
        "     border=no></a></td>"
        "</tr>",
        progress_icons[step],
        html_color(PROGRESS_DOWNLOAD, step, FALSE),
        U_("Downloading"),
        html_color(PROGRESS_DOWNLOAD, step, FALSE),
        U_("First, the requested packages are downloaded from their source"),
        html_color(PROGRESS_DOWNLOAD, step, TRUE),
        U_("Done"), download_done, 
        U_("Remaining"), (download_total - download_done),
        U_("Percent Complete"),
        (int) ((double) download_done / (double) download_total * 100.0),
        html_color(PROGRESS_VERIFY, step, FALSE),
        U_("Verifying"),
        html_color(PROGRESS_VERIFY, step, FALSE),
        _("Next, packages are verified to ensure cryptographic integrity"),
        html_color(PROGRESS_VERIFY, step, TRUE),
        U_("Done"), verify_done, 
        U_("Remaining"), (verify_total - verify_done),
        U_("Percent Complete"),
        (int) ((double) verify_done / (double) verify_total * 100.0));
    gtk_html_write(appinfo->download_html, stream, h, strlen(h));
    g_free(h);
    h = g_strdup_printf(
        "<tr valign=top>"
        "<td width=100>"
        "<font size=\"+1\" color=%s><b>%s</b></font></td>"
        "<td><font color=%s>%s</font>"
        "<font color=%s><table cellspacing=0 cellpadding=0><tr>"
        "<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>"
        "<td width=40>%s:</td>"
        "<td width=30><b>%d</b></td>"
        "<td width=85>&nbsp;&nbsp;&nbsp;&nbsp;%s:</td>"
        "<td width=30><b>%d</b></td>"
        "<td width=130>&nbsp;&nbsp;&nbsp;&nbsp;%s:</td>"
        "<td width=30><b>%d%%</b></td>"
        "</tr></table></font></td>"
        "<td align=right width=66>"
        "<a href=\"updater:transact_log\">"
        "<img src=\"file://"SHAREDIR"/info-off.png\" width=66 height=30"
        "     border=no></a></td>"
        "</tr>",
        html_color(PROGRESS_TRANSACT, step, FALSE),
        U_("Transacting"),
        html_color(PROGRESS_TRANSACT, step, FALSE),
        U_("The new packages are installed and old packages are removed."),
        html_color(PROGRESS_TRANSACT, step, TRUE),
        U_("Done"), transact_done,
        U_("Remaining"), (transact_total - transact_done),
        U_("Percent Complete"),
        (int) ((double) transact_done / (double) transact_total * 100.0));
    gtk_html_write(appinfo->download_html, stream, h, strlen(h));
    g_free(h);

    if (step == PROGRESS_WAIT_FOR_IT)
        h1 = U_("Finishing up...");
    else if (step == PROGRESS_FINISHED)
        h1 = U_("Installation has finished.");
    else {
        h1 = 
            U_("The packages you requested are being downloaded and installed "
              "on your system.");
    }

    h = g_strdup_printf(
        "</table>"        
        "<table width=100%% bgcolor=#000000 cellspacing=0 cellpadding=0>"        
        "<tr><td height=4>&nbsp;</td></tr>"
        "</table>"
        "<font color=#000000>"
        "<table width=100%% bgcolor=#e5e3e3 cellspacing=0 cellpadding=4"
        "       height=100%%>"
        "<tr>"
        "<td align=center colspan=2>%s</td>"
        "</tr>", h1);
    gtk_html_write(appinfo->download_html, stream, h, strlen(h));
    g_free(h);

    if (step == PROGRESS_DOWNLOAD) {
        h = g_strdup_printf(
            "<tr><td><b>%s</b></td></tr>"
            "<tr><td><object classid=\"single_progress\"></object></td></tr>",
            name ? name : U_("Downloading files"));
        gtk_html_write(appinfo->download_html, stream, h, strlen(h));
        g_free(h);
    }

    if (step != PROGRESS_WAIT_FOR_IT && step != PROGRESS_FINISHED) {
        h = g_strdup_printf (
            "<tr><td><b>%s</b></td></tr>"
            "<tr><td><object classid=\"total_progress\"></object></td></tr>",
            ((step == PROGRESS_DOWNLOAD) || !name) ? U_("Total") : name);
        gtk_html_write(appinfo->download_html, stream, h, strlen(h));
        g_free (h);
    }

    h = "</table></font></body></html>";
    gtk_html_write(appinfo->download_html, stream, h, strlen(h));
    gtk_html_end(appinfo->download_html, stream, GTK_HTML_STREAM_OK);

    if (step == PROGRESS_WAIT_FOR_IT) {
        gtk_progress_sucks_my_ass(GTK_PROGRESS(single));
        gtk_progress_sucks_my_ass(GTK_PROGRESS(tot));
    }

    if (step == PROGRESS_VERIFY) {
        gtk_progress_set_value (GTK_PROGRESS (tot), (gdouble) done);
    }
} /* rc_gui_set_progress_install */

static void
rc_gui_set_progress_remove(GUIInstallProgress step, int total, int done, 
                           char *name)
{
    GtkHTMLStream *stream;
    char *h;
    const char *h1;
    GtkWidget *single, *tot;

    static int remove_done;
    static int remove_total;

    single = gtk_object_get_data(
        GTK_OBJECT(appinfo->download_html), "single_progress");
    if (single->parent) {
        gtk_widget_ref(single);
        gtk_container_remove(GTK_CONTAINER(single->parent), single);
    }

    tot = gtk_object_get_data(
        GTK_OBJECT(appinfo->download_html), "total_progress");
    if (tot->parent) {
        gtk_widget_ref(tot);
        gtk_container_remove(GTK_CONTAINER(tot->parent), tot);
    }

    switch (step) {
    case PROGRESS_TRANSACT:
        if (total != 0) {
            remove_total = total;
        }
        remove_done = done;
        break;
    case PROGRESS_WAIT_FOR_IT:
    case PROGRESS_FINISHED:
        break;
    default:
        g_assert_not_reached();
    }

    stream = gtk_html_begin(appinfo->download_html);
    h = g_strdup_printf(
        "<html><body marginwidth=0 marginheight=0 bgcolor=#e5e3e3>"
        "<table width=100%% cellspacing=0 cellpadding=2 bgcolor=#ffffff>"
        "<tr valign=center>" /* valign=top */
        "<td rowspan=4 width=100>"
        "<img src=\"file://"SHAREDIR"/%s\" width=100 height=100>"
        "</td>"
        "<td width=100>"
        "<font size=\"+2\" color=%s><b>%s</b></font></td>"
        "<td><font color=%s>%s</font>"
        "<font color=%s><table cellspacing=0 cellpadding=0><tr>"
        "<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>"
        "<td width=40>%s:</td>"
        "<td width=30><b>%d</b></td>"
        "<td width=85>&nbsp;&nbsp;&nbsp;&nbsp;%s:</td>"
        "<td width=30><b>%d</b></td>"
        "<td width=130>&nbsp;&nbsp;&nbsp;&nbsp;%s:</td>"
        "<td width=30><b>%d%%</b></td>"
        "</tr></table></font></td>"
        "<td align=right width=66>"
        "<a href=\"updater:remove_log\">"
        "<img src=\"file://"SHAREDIR"/info-off.png\" width=66 height=30"
        "     border=no></a></td>"
        "</tr>",
        progress_icons[step],
        html_color(PROGRESS_TRANSACT, step, FALSE),
        U_("Removing"),
        html_color(PROGRESS_TRANSACT, step, FALSE),
        U_("The packages you selected are being removed from your system"),
        html_color(PROGRESS_TRANSACT, step, TRUE),
        U_("Done"), remove_done,
        U_("Remaining"), (remove_total - remove_done),
        U_("Percent Complete"),
        (int) ((double) remove_done / (double) remove_total * 100.0));
    gtk_html_write(appinfo->download_html, stream, h, strlen(h));
    g_free(h);

    if (step == PROGRESS_WAIT_FOR_IT)
        h1 = U_("Finishing up...");
    else if (step == PROGRESS_FINISHED)
        h1 = U_("Removal has finished.");
    else {
        h1 = U_("The packages you selected are being removed from your "
               "system.");
    }

    h = g_strdup_printf(
        "</table>"
        "<table width=100%% bgcolor=#000000 cellspacing=0 cellpadding=0>"
        "<tr><td height=4>&nbsp;</td></tr>"
        "</table>"
        "<table width=100%% bgcolor=#e5e3e3 cellspacing=0 cellpadding=4"
        "       height=100%%>"
        "<tr>"
        "<td align=center colspan=2>%s</td>"
        "</tr>", h1);
    gtk_html_write(appinfo->download_html, stream, h, strlen(h));
    g_free(h);

    if (step != PROGRESS_WAIT_FOR_IT && step != PROGRESS_FINISHED) {
        h = g_strdup_printf (
            "<tr><td><b>%s</b></td></tr>"
            "<tr><td><object classid=\"total_progress\"></object></td></tr>",
            name ? name : U_("Removing packages"));
        gtk_html_write(appinfo->download_html, stream, h, strlen(h));
        g_free (h);
    }

    h = "</table></body></html>";
    gtk_html_write(appinfo->download_html, stream, h, strlen(h));
    gtk_html_end(appinfo->download_html, stream, GTK_HTML_STREAM_OK);

    if (step == PROGRESS_WAIT_FOR_IT) {
        gtk_progress_sucks_my_ass(GTK_PROGRESS(single));
        gtk_progress_sucks_my_ass(GTK_PROGRESS(tot));
    }
} /* rc_gui_set_progress_remove */

void
rc_gui_set_progress(GUIInstallProgress step, int total, int done, char *name)
{
    if (install_type == GUI_INSTALL)
        rc_gui_set_progress_install(step, total, done, name);
    else if (install_type == GUI_REMOVE)
        rc_gui_set_progress_remove(step, total, done, name);
    else
        g_assert_not_reached();
} /* rc_gui_set_progress */

GUIInstallType
rc_gui_get_install_type(void)
{
    return install_type;
} /* rc_gui_get_install_type */

void
rc_gui_prepare_install_page(GUIInstallType type)
{
    GtkHTMLStream *stream;
    char *h;
    const char *t;

    install_type = type;

    stream = gtk_html_begin(GTK_HTML(GET_WIDGET("download_head_html")));
    if (type == GUI_INSTALL)
        t = U_("Package Installation");
    else
        t = U_("Package Removal");

    h = g_strdup_printf(
        "<html><body bgcolor=#000000 text=#ffffff marginwidth=0"
        "            marginheight=0>"
        "<table cellspacing=0 cellpadding=1 width=100%% border=0>"
        "<tr><td align=left>"
        "<img src=\"file://"SHAREDIR"/download.png\">"
        "</td><td align=left width=100%%>"
        "<font size=\"+2\"><b>%s</b></font>"
        "</td></tr></table></body></html>", t);
    gtk_html_write(GTK_HTML(GET_WIDGET("download_head_html")), stream, h,
                   strlen(h));
    gtk_html_end(GTK_HTML(GET_WIDGET("download_head_html")), stream,
                 GTK_HTML_STREAM_OK);

    g_free(h);
} /* rc_gui_prepare_install_page */

static gboolean
resize_progress_bars(gpointer data)
{
    gtk_widget_set_usize(
        GTK_WIDGET(data),
        GTK_WIDGET(appinfo->download_html)->allocation.width - 8,
        -2);

    return FALSE;
}

static void
size_allocate (GtkWidget *widget, GtkAllocation *allocation, gpointer data)
{
    /* So the problem here is that explicitly setting the usize causes us
     * to get into an infinite resizing loop and locks up the GUI.  My
     * (limited) understanding of GTK's sizing system is that when a widget
     * is resized, it queues a resize on its container as well, so it looks
     * like the HTML widget is telling the progress bar to resize, which is
     * telling the HTML widget to resize, which is telling the progress bar
     * to resize, etc.  Actually resizing the widget in an idle handler
     * seems to make it work, as the HTML widget will already be the correct
     * size by the time it requests a resize from the progress bar.
     */

    gtk_idle_add(resize_progress_bars, data);

#if 0
    gtk_widget_set_usize (GTK_WIDGET (data), allocation->width - 8, -1);
#endif
}

void
rc_gui_init_install_page(void)
{
    GtkWidget *w;

    w = gtk_progress_bar_new();
    gtk_progress_set_show_text(GTK_PROGRESS(w), TRUE);
    gtk_widget_show(w);
    gtk_object_set_data(
        GTK_OBJECT(appinfo->download_html), "single_progress", w);
    gtk_signal_connect (
        GTK_OBJECT (appinfo->download_html), "size-allocate",
        GTK_SIGNAL_FUNC (size_allocate), (gpointer) w);

    w = gtk_progress_bar_new();
    gtk_progress_set_show_text(GTK_PROGRESS(w), TRUE);
    gtk_widget_show(w);
    gtk_object_set_data(
        GTK_OBJECT(appinfo->download_html), "total_progress", w);
    gtk_signal_connect (
        GTK_OBJECT (appinfo->download_html), "size-allocate",
        GTK_SIGNAL_FUNC (size_allocate), (gpointer) w);
} /* rc_gui_init_install_page */
