/* -*- Mode: C; tab-width: 4; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
 * gui.h: GUI structs
 *
 * Copyright (C) 2000, 2001 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __REDCARPET_GUI_H
#define __REDCARPET_GUI_H

#include <glib.h>
#include <glade/glade.h>
#include <gtkhtml/gtkhtml.h>

#include <gal/shortcut-bar/e-shortcut-model.h>

#include "libredcarpet.h"

#include "gui-cache.h"
#include "gui-channel.h"
#include "gui-log.h"
#include "gui-rdf.h"

#include "e-splash.h"

typedef enum {
    PAGE_UPDATES,
    PAGE_INSTALLED,
    PAGE_AVAILABLE,
    PAGE_PREFERENCES,
    PAGE_INFO,
    PAGE_SEARCH
} MinorNotebookPage;

typedef enum {
    PAGE_SUMMARY,
    PAGE_DETAILS
} MedianNotebookPage;

typedef enum {
    PAGE_CHANNELS,
    PAGE_DOWNLOAD,
    PAGE_ERROR,
    PAGE_VERIFY,
    PAGE_MISC
} MainNotebookPage;

typedef enum {
    ACTION_SUMMARY,
    ACTION_NEWS,
    ACTION_FEATURED,
    ACTION_SUBSCRIBED,
    ACTION_UNSUBSCRIBED,
    ACTION_UPDATE,
    ACTION_INSTALL,
    ACTION_REMOVE,
    ACTION_PREFERENCES,
    ACTION_INFO,
    ACTION_SEARCH,
    ACTION_ABOUT
} ActionType;

typedef enum {
    SHORTCUT_NAVIGATION,
    SHORTCUT_SHORTCUTS
} ShortcutGroupType;

typedef enum {
    REFRESH_CHANNEL,
    REFRESH_SUMMARY,
    REFRESH_NEWS
} RefreshType;

typedef enum {
    SELECT_NEITHER   = 0,
    SELECT_ITEM_ALL  = 1 << 0,
    SELECT_ITEM_NONE = 1 << 1,
} SelectType;

#define CKINSTALL_IMAGE_URL   "file://"SHAREDIR"/ckin.png"
#define CKREMOVE_IMAGE_URL    "file://"SHAREDIR"/ckout.png"
#define CKNOTHING_IMAGE_URL   "file://"SHAREDIR"/cknone.png"

typedef enum {
    LOG_DOWNLOAD,
    LOG_VERIFY,
    LOG_TRANSACT,
    LOG_REMOVE,
    LOG_LAST /* Invalid */
} LogIndex;

typedef struct _AppInfo    AppInfo;

struct _AppInfo {
    /* Glade object */
    GladeXML *xml;

    /* HTML Widgets */
    GtkHTML *update_html;
    GtkHTML *installed_html;
    GtkHTML *available_html;
    GtkHTML *info_html;
    GtkHTML *exec_head;
    GtkHTML *exec_body;
    GtkHTML *action_html;
    GtkHTML *package_html;
    GtkHTML *download_html;

    MinorNotebookPage current_page;
    GUILog *log[LOG_LAST];

    GtkHTMLStream *info_stream;
    
    GtkWidget *bar;
    EShortcutModel *model;

    GUIChannelSList *channel_list;
    /* should be NULL when the exec summary is shown */
    GUIChannel *current_channel; 

    /* Packman object */
    RCPackman *pman;

    /* The system packages */
    RCPackageSList *system_packages;

    /* Are we using the Priority Updates/Partnernet system? */
    gboolean priority;

    /* Are we root? */
    gboolean not_root;

    /* Regular user's uid */
    uid_t uid;
    
    /* What was the last thing searched for? */
    char *last_search;

    /* Case sensitive search */
    gboolean case_sensitive;

    /* The news info */
    RDFSummary *news;

    /* The splash screen */
    ESplash *splash;

    /* There are necessary updates */
    RCPackageSList *necessary;

    /* used for restarting */
    char * const *argv;

    /* Transfer caches */
    GUICache *package_cache;
    GUICache *icon_cache;
    GUICache *normal_cache;

    /* RCD overrides */
    char *rcd_host_url;
    gboolean rcd_priority;
};

extern AppInfo *appinfo;

#define GET_WIDGET(x) ((glade_xml_get_widget(appinfo->xml, x)))

/* Misc common functions */
void rc_gui_fetch_all_icons(void);
void rc_gui_fetch_icon(GUIChannel *channel);

void rc_gui_refresh_all_channels(void);

char *rc_gui_get_xauthority_file(void);

void rc_quit (void);

#endif /* __REDCARPET_GUI_H */
